import time, urllib, logging
from pasteob import *

__all__ = ['log_mw']

_default_format = ('%(REMOTE_ADDR)s - %(REMOTE_USER)s [%(time)s] '
              '"%(REQUEST_METHOD)s %(REQUEST_URI)s %(HTTP_VERSION)s" '
              '%(status)d %(bytes)s "%(HTTP_REFERER)s" "%(HTTP_USER_AGENT)s"')


class log_mw(object):
    """
        This logging middleware will log all requests as they go through.
        They are, by default, sent to a logger named ``'wsgi'`` at the
        INFO level.

        If ``setup_console_handler`` is true, then messages for the named
        logger will be sent to the console.
    """

    def __init__(self, application,
                 logger=None,
                 format=_default_format,
                 logging_level=logging.INFO,
                 setup_console_handler=True):
        self.format = format
        self.application = application
        self.logging_level = logging_level
        if logger is None:
            logger = logging.getLogger('wsgi')
            logger.setLevel(logging.DEBUG)
            if setup_console_handler:
                console = logging.StreamHandler()
                console.setLevel(logging.DEBUG)
                # We need to control the exact format:
                console.setFormatter(logging.Formatter('%(message)s'))
                logger.addHandler(console)
                logger.propagate = False
        self.logger = logger

    @webob_wrap
    def __call__(self, req):
        start = time.localtime()
        resp = req.get_response(self.application)
        message = self.format_msg(req, resp, start)
        self.logger.log(self.logging_level, message)
        return resp

    def format_msg(self, req, resp, start):
        bytes = resp.content_length
        if bytes is None:
            bytes = '-'
        timezone = time.altzone if time.daylight else time.timezone
        offset = " %+0.4d" % (timezone / 60 / 60 * -100)
        return self.format % {
            'REMOTE_ADDR': or_dash(req.remote_addr),
            'REMOTE_USER': or_dash(req.remote_user),
            'REQUEST_METHOD': req.method,
            'REQUEST_URI': req.path_qs,
            'HTTP_VERSION': req.environ.get('SERVER_PROTOCOL'),
            'time': time.strftime('%d/%b/%Y:%H:%M:%S', start) + offset,
            'status': resp.status_int,
            'bytes': bytes,
            'HTTP_REFERER': or_dash(req.referer),
            'HTTP_USER_AGENT': or_dash(req.user_agent),
        }

def or_dash(val):
    return val or '-'
